require_relative 'cloud_runtime'
require 'uu/os/cmd/command_client'
require 'uu/os/security/session'

module UU
  module CloudRuntime
    # Resource represents a service or other resource assigned to an
    # application on demand, especially in the area of persistence (e.g.
    # MongoDB, PostgreSQL, ...) or messaging.
    class Resource < ::UU::CloudRuntime::RuntimeBaseClass

      # Supported resource types.
      class ResourceType
        # Message bus resource.
        MSG_BUS = 'MSG_BUS'
        # Data store resource - object store (OSID), binary store (BSID).
        DATA_STORE = 'DATA_STORE'
      end

      # Remote service path.
      # @private
      PATH = '/uu-cloud-runtime/Resource'
      _init_command_client(PATH)

      # Creates a new instance of +Resource+.
      #
      # @param session [UU::OS::Security::Session] Instance of session representing authenticated user
      #   (can be obtained with {UU::OS::Security::Session#current_session} method).
      def initialize(session = nil)
        @client = UU::OS::CMD::CommandClient.new(PATH, session)
      end

      # Returns a paged list of Resources filterable by resource type.
      #
      # @example Command call
      #   require 'uu_c3'
      #   UU::OS::Security::Session.login('...')
      #   tid_asid = '123-35830958093'
      #
      #   resource = UU::CloudRuntime::Resource.new(UU::OS::Security::Session.current_session)
      #   list = resource.get_resource_list(
      #     tid_asid,
      #     :pageIndex => 0,
      #     :pageSize => 1000
      #   )
      #
      # @example Command response
      #   {
      #     :totalSize => 1,
      #     :criteria => {
      #       :pageSize => 1000,
      #       :pageIndex => 0,
      #       :resourceType => null
      #     },
      #     :pageEntries => [
      #       {
      #         :resourceType => "DATA_STORE",
      #         :id => "v3dphou7k303n2gzfklsxzon9ymlo817",
      #         :connectionUrl => "mongodb://v3dphou7k303n2gzfklsxzon9ymlo817__b933l3ojvini92p2o9ongo-o-s.eu-n-1.cloud.plus4u.net:27017/v3dphou7k303n2gzfklsxzon9ymlo817"
      #       },
      #       {
      #         :resourceType => "MSG_BUS",
      #         :messageBusUri => "ues:OT-BT:MESSAGE_BUS_001",
      #         :queuePairs => ["QUEUE_096", "QUEUE_099"]
      #       }
      #     ]
      #   }
      #
      # @param app_deployment_uri_or_tid_asid [String, UU::OS::UESURI] App deployment uri or tid-asid.
      # @param parameters [Hash] Filtering criteria.
      # @option parameters [Fixnum] pageIndex Index of page of the result list.
      # @option parameters [Fixnum] pageSize Maximum records count per page.
      # @option parameters [UU::CloudRuntime::Resource::ResourceType, String] resourceType
      #   Filter records by resource type. May be +nil+.
      # @return [Hash] Structured hash.
      def get_resource_list(app_deployment_uri_or_tid_asid, parameters = {})
        @client.invoke(:getResourceList, app_deployment_uri_or_tid_asid, parameters: parameters)
      end

      # Returns a paged list of Resources filterable by resource type.
      # This method is a convenience shortcut for the {#get_resource_list}
      # instance method.
      #
      # @param app_deployment_uri_or_tid_asid [String, UU::OS::UESURI, String] App deployment uri or tid-asid.
      # @param parameters [Hash] Filtering criteria.
      # @option parameters [Fixnum] pageIndex Index of page of the result list.
      # @option parameters [Fixnum] pageSize Maximum records count per page
      # @option parameters [UU::CloudRuntime::Resource::ResourceType, String] resourceType
      #   Filter records by resource type. May be +nil+.
      # @return [Hash] Structured hash.
      # @see .get_resource_list
      def self.get_resource_list(app_deployment_uri_or_tid_asid, parameters = {})
        self.new(UU::OS::Security::Session.current_session).get_resource_list(app_deployment_uri_or_tid_asid, parameters)
      end

    end
  end
end
